#############################

Multi.Dep_WavPer.SMC_complete<-function(data.mat, no.X, init.vec=NULL, N=100, B=100, temp.sched=NULL, hyper.vec, prop.var.vec, filter.number=1, filter.family="DaubExPhase"){
#Function to perform the SMC samplers with respect to multivariate time process from the wavelet periodogram
#Need to estimate the posterior of parameters (TPs, state dependent power)
	#The state dependent power affects the covariance structure of the wavelet processes

#data.mat=matrix containing the time series (periodogram), each row represents a different scale, each column represents a different time
	#Going further down the rows of matrix corresponds to coarser scale processes
#no.X= Number of states assumed in our HMM
#init.vec=The initialisation probability vector for our HMM
#N=Number of particles/samples in our SMC samplers approximations
#B=Number of iterations from intrumental (prior) to target (posterior) distribution
#temp.sched= non-decreasing temperature scehdule describing the sequence of distributions
#hyper.vec =hyper parameters used in prior sampling
#prop.var.vec =proposal variance vector for mutation step
#filter.number, filter.family correspond to the mother wavelet we are using for analysis (see wavethresh)

if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X} #If an initial vector is not specified
if(is.null(temp.sched)==TRUE){temp.sched<-seq(0, 1, len=B)} #Temperature schedule if one isn't provided (linear schedule)

#Preamble
data.T<-ncol(data.mat); data.J<-nrow(data.mat) #Length of time series and number of scales considered
filter.info<-discrete.wavelets(J=data.J, filter.number=filter.number, filter.family=filter.family)  #The necessary wavelet filtering coefficients associated with analysis
ramping.len<-length(filter.info[[data.J]]) #Support of the largest wavelet 
order.m<-2*(ramping.len-1) #Order of the underlying MC we use

#Initialisation, sample from prior and compute the log lik of associated samples
Initial.info<-Multi.Dep_WavPer.SMC_initial(N=N, hyper.vec=hyper.vec, data.mat=data.mat, init.vec=init.vec, no.X=no.X, filter.info=filter.info, order.m=order.m)

theta.list<-Initial.info[[1]] #The samples
loglik.vec<-Initial.info[[2]]	#The associated log-lik
nweights.vec<-Initial.info[[3]]	#The associated weights

init.samples.mat<-matrix(theta.list, nrow=N, ncol=1, byrow=F)#Matrix where each row represents a different sample
    #The idea is store all samples in an array where the 3rd dimension represents the iterations step

#Storage
particle.storage.array<-array(init.samples.mat, dim=c(N,1,B))
loglik.mat<-t(as.matrix(loglik.vec)) #Log-lik matrix
weights.mat<-t(as.matrix(nweights.vec))	#Normalised weights matrix
resample.vec<-accep.prob.mat<-NULL	#Resampling times vector and the acceptance probabilities
NC.vec<-NULL #Vector to store normalising constants of weights. This is to compute the approximation of the normalising constant of the posterior (which turns out to be the posterior)
const.vec<-NULL

#Mutation step, 
for(iter in 2:B){
print(iter)
temperature<-temp.sched[iter] #Current associated temperature
curr.part.mat<-particle.storage.array[,,iter-1] #The current set samples we will mutate
curr.loglik.vec<-loglik.mat[iter-1,] #The associated log-liks
curr.weight.vec<-weights.mat[iter-1,]	#Associated weights

#Weights calculation, done before the mutation since the weights don't depend on the new mutated particle.
base<-curr.loglik.vec *(temperature-temp.sched[iter-1])
inc.weight<-exp(base-max(base)) #The incremental weight
const<--max(base)
curr.weight.vec<-curr.weight.vec*inc.weight #The new un-normalised weights
Norm<-sum(curr.weight.vec) #The normalising constant 
curr.weight.vec<-curr.weight.vec/Norm #Normalised weights

#Resample
ESS<-1/sum(curr.weight.vec^2) #Effective sample size
if(ESS < (0.5*N)){ #Resampling   condition
Resample_index<-System_Resample(index.vec=1:N, w.vec=curr.weight.vec) #Indices from resampling procedure, resampling achieved by systematic resampling
#if(length(Resample_index)!=N){browser()}
curr.part.mat<-curr.part.mat[Resample_index] #Extract the corresponding resampled samples
curr.loglik.vec<-curr.loglik.vec[Resample_index] #...and the corresponding log likelihoods
curr.weight.vec<-rep(1/N, N) #Renormalise the resampled samples
Norm<-sum(curr.weight.vec) #The normalising constant 
resample.vec<-c(resample.vec, iter) #The associated resampling time
}

NC.vec<-c(NC.vec, log(Norm)-const) #The actual normalising constant for MLL approximation
weights.mat<-rbind(weights.mat, curr.weight.vec) #Add the associated new weights for this iteration

#Actual mutation procedure, performed on each sample individually. This could be parallelised
for(part in 1:N){
curr.part<-curr.part.mat[[part]]; curr.loglik<-curr.loglik.vec[part]
#The current sample and the log-lik in question
mutation_info<-Multi.Dep_WavPer.SMC_mutate(curr.sample=curr.part, curr.loglik=curr.loglik, hyper.vec=hyper.vec, prop.var.vec=prop.var.vec/(iter-1), temperature=temperature, data.mat=data.mat, init.vec=init.vec, filter.info=filter.info, order.m=order.m) #Mutation information

curr.part.mat[[part]]<-mutation_info[[1]] #The new mutated particle
curr.loglik.vec[part]<-mutation_info[[2]] #The new likelihood
}

#Add everything to general storage
particle.storage.array[,,iter]<-curr.part.mat
loglik.mat<-rbind(loglik.mat, curr.loglik.vec)
}

list(curr.part.mat, curr.weight.vec, curr.loglik.vec, NC.vec, "Resample times"=resample.vec)
#curr.part.mat=the samples approximating the target posterior distribution
#curr.weight.vec=the associated weights of the samples
#curr.loglik.vec=the associated log-lik of the samples
#NC.vec= the vector of normalising constants for the weights
#resample.vec= the vector of resampling times
}

###################
Multi.Dep_WavPer.SMC_initial<-function(N, hyper.vec, data.mat, init.vec=NULL, no.X, filter.info, order.m){
#Function to perfrom initialisation, sampling from the prior and the associated log-lik and weights
	#Parameters to sample are the TP and the state dependent power

#N=Number of samples to sample from prior
#hyper.vec=hyperparameters of the associated prior we are sampling from
#data.mat=the wavelet periodogram considered in matrix form. Columns are time, rows are scales
#init.vec=probability initialisation vector for HMM
#no.X=number of underlying states assumed in HMM
#filter.info=information associated with mother wavlet
#order.m=order of the underlying HMM
#Preamble
data.T<-ncol(data.mat); data.J<-nrow(data.mat) #Length of time series and number of scales considered
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X} #If an initial vector is not specified
init.theta.list<-NULL
init.loglik.vec<-NULL
N.star<-N
while(N.star!=0){ #For debugging purposes in case we get bad particles with bad log-liks
Raw.init.samples<-Multi.Dep_WavPer.SMC_sample(N=N.star, hyper.vec=hyper.vec, no.X=no.X, data.J=data.J)
	#Sample some samples from the prior
Raw.log.lik<-sapply(Raw.init.samples, FUN=Multi.Dep_WavPer.LogLik, data.mat=data.mat, init.vec=init.vec, filter.info=filter.info, order.m=order.m)
	#Compute the associated log-lik of these samples
bad.index<-which(is.nan(Raw.log.lik)==TRUE) #Which are the bad samples with undefined log-lik
if(length(bad.index)!=0){ #When bad samples exist
init.theta.list<-c(init.theta.list, Raw.init.samples[-bad.index]) #Take through only the good samples
init.loglik.vec<-c(init.loglik.vec, Raw.log.lik[-bad.index]) #And the associated log-lik
N.good<-N.star-length(bad.index) #Number of good samples
N.star<-N.star-N.good #How many new samples to sample in the next iteration
}
else{
init.theta.list<-c(init.theta.list, Raw.init.samples) #Samples
init.loglik.vec<-c(init.loglik.vec, Raw.log.lik) #Associated log-lik
N.star<-0
}
}
list("I_Samples"=init.theta.list, "I_LogLik"=init.loglik.vec, "I_Weights"=rep(1/N, N))
#"I_Samples"=Samples from the prior distribution
#"I_LogLik"=Associated log-lik of the samples
#"I_Weights" = Associated weights of the samples
}
#####################
Multi.Dep_WavPer.SMC_sample<-function(N, hyper.vec, no.X, data.J){
#Function to perform the prior sampling part of intialisation

#N= number of samples to sample from the prior
#hyper.vec =hyper parameters for prior
#no.X = Number of states assumed in our HMM
#data.J=number of scale processes we are considering in our analysis

tp.hyper<-hyper.vec[1] #Main hyperparameter for TP
power.hyper<-hyper.vec[-1] #Main hyperparameters for the power constants

#Transition Probability Matrix
tp.hyper.mat<-matrix(1, nrow=no.X, ncol=no.X) #Create a matrix corresponding to the hyperparameters for each TP vector
diag(tp.hyper.mat)<-tp.hyper #To favour persistency

tp.init.array<-array(1, dim=c(no.X, N, no.X)) #Array to store the initisation transition probability vectors
for(row in 1:no.X){
tp.init.array[,,row]<-dirchlet.sample(shape.vec=tp.hyper.mat[row,], no.X=no.X, no.samp=N)
	#Sample some row transition probabilities with respect to corresponding Dirichlet prior
}

#State dependent power configuration, deal with the inverse of the power in the SMC
#Analogous to precision in usual parameter analysis
	#Each state corresponds to a different power configuration over the scales of interest.
power.samples<-rgamma(no.X*data.J*N, shape=power.hyper[1], scale=power.hyper[2])
power.init.array<-array(power.samples, dim= c(no.X, data.J, N)) #Array containing the scaling samples
	#Dim 1 (row) = state
	#Dim 2 (column) = scale
	#Dim 3 = particle

#Putting our particles together
theta.list<-vector("list", N)
X.SS<-1:no.X
for(part in 1:N){
tp.mat<-matrix(tp.init.array[,part,], nrow=no.X, ncol=no.X, byrow=T, dimnames=list(X.SS, X.SS))
	#Create the transition matrix we've created thus far

theta<-list("TP matrix"=tp.mat, "Inverse Power Constants"=matrix(power.init.array[,,part], nrow=no.X, ncol=data.J))
#Sample containing the TP matrix and inverse power constants matrix.
theta.list[[part]]<-theta
}
theta.list #List containing the samples from the prior.
}


#####################
Multi.Dep_WavPer.SMC_mutate<-function(curr.sample, curr.loglik, hyper.vec, prop.var.vec, temperature, data.mat, init.vec, filter.info, order.m){
#Mutation part of the algorithn

#curr.sample=current particle that will are going to mutatue
#curr.loglik=current loglik associated with the particle
#hyper.vec=hyper parameters for priors of parameters
#prop.var.vec= proposal variances for parameters
#temperature =current temperature, which corresponds to the distribution we are at
#data.mat=raw wavelet periodogram
#init.vec=probability initialisation vector of HMM
#filter.info=filter information associated with NDWT used to achieve the raw wavelet periodogram
#order.m=order of dependency induced in HMM

#Parameters partitioning
curr.TP.mat<-curr.sample[[1]]	#Current TP matrix
no.X<-nrow(curr.TP.mat)	#No of states assumed in HMM
data.J<-nrow(data.mat) #Number of periodogram scales considered

#Hyper parameter partitioning
tp.hyper<-hyper.vec[1] #Hyperparameters associated with TP
	tp.hyper.mat<-matrix(1, nrow=no.X, ncol=no.X); diag(tp.hyper.mat)<-tp.hyper
power.hyper<-hyper.vec[-1]

#Proposal variances for TP and power
tp.prop.var<-prop.var.vec[1]; power.prop.var<-prop.var.vec[2]

#Transition probability mutation based on current TP matrix via RWMH on logit scale on each row vector
prop.TP.mat<-TPmat.proposal.logit(curr.TPmat=curr.TP.mat, prop.var=tp.prop.var)

#Sett everything up
upd.sample<-curr.sample
upd.loglik<-curr.loglik

#Testing each row separately
for(row.index in 1:no.X){
temp.prop.TP.mat<-upd.sample[[1]] #Take the most recent value of TP matric
temp.prop.TP.mat[row.index,]<-prop.TP.mat[row.index, ] #Add the proposal TP vector to the must recent version of the TP matrix

acc_rej.info<-Multi.Dep_WavPer.SMC_TPmat_AccRej(curr.theta=upd.sample, curr.li=upd.loglik, prop.tp.mat=temp.prop.TP.mat, tp.hyper.prior=tp.hyper.mat[row.index,] , row.index=row.index,  prop.var=tp.prop.var, data.mat=data.mat, init.vec=init.vec, temperature=temperature, filter.info=filter.info, order.m=order.m)
#Perform the accpetance rejection part

upd.sample<-acc_rej.info[[1]] #The accepted sample
upd.loglik<-acc_rej.info[[2]]	#And the corresponding log-lik
}

#Inverse Power matrix
curr.power.mat<-curr.sample[[2]] #Most recent value of the power matrix
prop.power.mat<-exp(log(curr.power.mat)+rnorm(no.X*data.J, mean=0, sd=sqrt(power.prop.var)) )
	#Perform independent proposals on each of the scaling coefficients
	#Achieved by RWMH on log scale

for(element in 1:(no.X)){ #Acc/Rej each state dependent power vector across all scales 
acc_rej.info<-Multi.Dep_WavPer.SMC_Power_AccRej(curr.theta=upd.sample, curr.li=upd.loglik, prop.power.const=prop.power.mat[element,], state=element, power.hyper.prior=power.hyper, data.mat=data.mat, init.vec=init.vec, temperature=temperature, filter.info=filter.info, order.m=order.m)

upd.sample<-acc_rej.info[[1]]#Accepted sample
upd.loglik<-acc_rej.info[[2]]#Associated log-lik
}

#Output
list(upd.sample, upd.loglik)
#upd.sample = the mutated sample based on RWMH move
#upd.loglik=the associated log-lik of the mutated sample.
}

#######################
Multi.Dep_WavPer.SMC_TPmat_AccRej<-function(curr.theta, curr.li, prop.tp.mat, tp.hyper.prior , row.index,  prop.var, data.mat, init.vec, temperature, filter.info, order.m){
#curr.theta=current sample to mutate
#curr.li=log-lik associated with current sample
#prop.tp.mat= the proposal TP matrix
#tp.hyper.prior=hyperparameter associated with TP
#row.index=row index that we are testing the proposal against
#prop.var=the proposal variance
#data.mat=raw wavelet periodogram
#init.vec=probability initialisation vector
#temperature=temperature at current iteration of SMC samplers
#filter.info=filter information associated with wavelet transform
#order.m=order of dependency in HMM

#Acc/ Rej mechanism for the transition matrix
prop.theta<-curr.theta
prop.theta[[1]]<-prop.tp.mat #Create the proposal theta based on the proposal TP mat
curr.tp.mat<-curr.theta[[1]] #Current TP mat

#Transition Probability Vector under scrutiny
curr.tp.vec<-curr.tp.mat[row.index,]
prop.tp.vec<-prop.tp.mat[row.index,]

#Prior component
prop.prior<-dirchlet.log.density(xvec=prop.tp.vec, shape.vec=tp.hyper.prior) #Prior density under proposal
if(is.nan(prop.prior)==TRUE){list(curr.theta, curr.li)}
else{curr.prior<-dirchlet.log.density(xvec=curr.tp.vec, shape.vec=tp.hyper.prior) #Prior density under current
prior.comp<-prop.prior-curr.prior #Prior component of acceptance ratio

#Log-Lik component
prop.li<-Multi.Dep_WavPer.LogLik(data.mat=data.mat, theta=prop.theta, init.vec=init.vec, filter.info=filter.info, order.m=order.m) #Log-lik under proposal
if(is.nan(prop.li)==TRUE){list(curr.theta, curr.li)}
else{loglik.comp<-temperature*(prop.li-curr.li) #Log-lik component of acceptance ratio

#Jacob compononent of acceptance ratio since we have a change in variable
prop.jacob<--sum(log(curr.tp.vec)) #Jumping from proposal, to current 
curr.jacob<--sum(log(prop.tp.vec)) #Jumping from current, to proposal
jacob.comp<-prop.jacob-curr.jacob 

alpha<-min(0, prior.comp+loglik.comp+jacob.comp)
uno<-log(runif(1))
if(uno<alpha){list(prop.theta, prop.li)}
else{list(curr.theta, curr.li)}
#Returns a sample and the associated log-lik based on the acceptance rejection mechanism
}}
}

################
Multi.Dep_WavPer.SMC_Power_AccRej<-function(curr.theta, curr.li, prop.power.const, state, power.hyper.prior , prop.var, data.mat, init.vec, temperature, filter.info, order.m){
#Acc/ Rej mechanism for the power matrix

#curr.theta=current sample to mutate
#curr.li=log-lik associated with current sample
#prop.power.const =  the proposal inverse power mat
#state=the state powers we are testing
#power.hyper.prior=hyperparameter associated with power
#prop.var=the proposal variance associated with power
#data.mat=raw wavelet periodogram
#init.vec=probability initialisation vector
#temperature=temperature at current iteration of SMC samplers
#filter.info=filter information associated with wavelet transform
#order.m=order of dependency in HMM

curr.power.mat<-curr.theta[[2]]

prop.theta<-curr.theta
prop.power.mat<-curr.power.mat
prop.power.mat[state,]<-prop.power.const
prop.theta[[2]]<-prop.power.mat #Create the proposal theta based on the proposal power mat

#These powers are in fact the inverse power

#Power constant under scrutiny
curr.power.const<-curr.power.mat[state,]
prop.power.const<-prop.power.mat[state,]

#Prior component
prop.prior<-sum(dgamma(prop.power.const, shape=power.hyper.prior[1], scale=power.hyper.prior[2], log=T)) #Prior density under proposal
if(is.nan(prop.prior)==TRUE){list(curr.theta, curr.li)}
else{curr.prior<-sum(dgamma(curr.power.const, shape=power.hyper.prior[1], scale=power.hyper.prior[2], log=T))#Prior density under current
prior.comp<-prop.prior-curr.prior #Prior compoent of acceptance ratio

#Log-Lik component
prop.li<-Multi.Dep_WavPer.LogLik(data.mat=data.mat, theta=prop.theta, init.vec=init.vec, filter.info=filter.info, order.m=order.m)#Log-lik under the proposal
if(is.nan(prop.li)==TRUE){list(curr.theta, curr.li)}
else{loglik.comp<-temperature*(prop.li-curr.li)#Log-lik component of acceptance ratio

#Jacob compononent

prop.jacob<--log(curr.power.const)#Jumping from proposal, to current 
curr.jacob<--log(prop.power.const) #Jumping from current, to proposal
jacob.comp<-prop.jacob-curr.jacob #Change of variables part of the acceptance ratio

alpha<-min(0, prior.comp+loglik.comp+jacob.comp)#Acceptance probability on log scale
uno<-log(runif(1))
if(uno<alpha){list(prop.theta, prop.li)}
else{list(curr.theta, curr.li)} #Sample and association log-lik
}}
}

################
Multi.Dep_WavPer.SMC_partial<-function(data.mat, no.X, init.vec=NULL, B=100, temp.sched=NULL, hyper.vec, prop.var.vec, filter.number=1, filter.family="DaubExPhase", start_iter, end_iter, start_part_list, start_loglik_vec, start_weights_vec, start_MLL_vec, start_resample){
#Function to perform the SMC samplers with respect to multivariate time process from the wavelet periodogram
#Partial ersion in that we consider parts of the samplers in case we need to speed it up on a cluster
#Need to estimate the posterior of parameters (TPs, state dependent power)                                        
	#The state dependent power affects the covariance structure of the wavelet processes

#Data.mat=matrix containing the time series, each row represents a different scale, each column represents a different time
	#Going further down the rows corresponds to coarser scale processes
#no.x= Number of states assumed in our HMM
#init.vec=The initialisation vector for our HMM
#N=Number of particles in our SMC samplers approximations
#B=Number of iterations from intrumental to target distribution
#temp.sched= non-decreasing temperature scehdule describing the sequence of distributions
#hyper.vec =hyper parameters used in prior sampling
#prop.var.vec =proposal variance vector
#filter.number, filter.family correspond to the mother wavelet we are using for analysis

#start_iter=the start iteration
#end_iter=end iteration 
#start_part_list=the starting sample list of SMC samplers
#start_loglik_vec=starting log lik of samples
#start_weights_vec=starting weights of samples
#start_MLL_vec=NC constant vector carried on from before
#start_resample=resampling vector

if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X} #If an initial vector is not specified
if(is.null(temp.sched)==TRUE){temp.sched<-seq(0, 1, len=B)} #Temperature schedule if one isn't provided
N<-length(start_part_list) #No samples involved in the SMC samplers

#Preamble
data.T<-ncol(data.mat); data.J<-nrow(data.mat) #Length of time series and number of wavelet scales analysed
filter.info<-discrete.wavelets(J=data.J, filter.number=filter.number, filter.family=filter.family)  #The necessary wavelet filtering information needed for analysis
order.m<-2*((2*filter.number)^data.J)-1 #Order of the underlying MC we use

#Code/objects specific to the partinioning of the code
sec.B<-end_iter-start_iter+1 #The number of iterations that we are going to perform in this job
#sec.temp.sched<-temp.sched[(start_iter-1):end_iter]

#Storage
particle.storage.array<-array(start_part_list, dim=c(N,1,sec.B+1))
  particle.storage.array[,,1]<-start_part_list #Storage for the samples
loglik.mat<-t(matrix(start_loglik_vec)) #Log-lik storage
weights.mat<-t(matrix(start_weights_vec)) #Weights storage
resample.vec<-start_resample #Resampling vector
accep.prob.mat<-NULL #Acceptance probability mat
NC.vec<-start_MLL_vec #Vector containing the normalising constants for the weights
const.vec<-NULL

#Mutation loop
for(iter in 1:sec.B){
real.iter<-c(start_iter:end_iter)[iter] #The corresponding iteration in the complete algorithm
temperature<-temp.sched[real.iter] #Current associated temperature
curr.part.mat<-particle.storage.array[,,(iter-1)+1] #Current samples to mutate
curr.loglik.vec<-loglik.mat[(iter-1)+1,] #Current associated log-lik, Because things are out by one slightly
curr.weight.vec<-weights.mat[(iter-1)+1,] #Current associated weights

#Weights calculation, done before the mutation since the weights don't depend on the new mutated particle.
base<-curr.loglik.vec *(temperature-temp.sched[real.iter-1])
inc.weight<-exp(base-max(base)) #The incremental weight
const<--max(base)
curr.weight.vec<-curr.weight.vec*inc.weight #The new un-normalised weights
Norm<-sum(curr.weight.vec) #The normalising constant 
curr.weight.vec<-curr.weight.vec/Norm #Normalised weights

#Resample
ESS<-1/sum(curr.weight.vec^2) #To determine if resampling is necessary
if(ESS < (0.5*N)){
Resample_index<-System_Resample(index.vec=1:N, w.vec=curr.weight.vec) #Indices from resampling procedure based on systematic resampling
if(length(Resample_index)!=N){browser()}
curr.part.mat<-curr.part.mat[Resample_index] #Extract the corresponding resampled samples
curr.loglik.vec<-curr.loglik.vec[Resample_index] #...and the corresponding log likelihoods
curr.weight.vec<-rep(1/N, N) #Renormalise weights after resampling
Norm<-sum(curr.weight.vec) #The normalising constant 
resample.vec<-c(resample.vec, real.iter) #Add iteration to resampling times
}

#Norm<-sum(curr.weight.vec) #The normalising constant 
#curr.weight.vec<-curr.weight.vec/Norm #Normalised weights
NC.vec<-c(NC.vec, log(Norm)-const) #The actual normalising constant for MLL approximation
weights.mat<-rbind(weights.mat, curr.weight.vec) #Add weights to storage

#Actual mutation procedure
for(part in 1:N){
curr.part<-curr.part.mat[[part]]; curr.loglik<-curr.loglik.vec[part]
mutation_info<-Multi.Dep_WavPer.SMC_mutate(curr.sample=curr.part, curr.loglik=curr.loglik, hyper.vec=hyper.vec, prop.var.vec=prop.var.vec/(real.iter-1), temperature=temperature, data.mat=data.mat, init.vec=init.vec, filter.info=filter.info, order.m=order.m) #Mutate the current sample

curr.part.mat[[part]]<-mutation_info[[1]] #The new mutated particle
curr.loglik.vec[part]<-mutation_info[[2]] #The new likelihood
}

#Add everything to general storage
particle.storage.array[,,(iter)+1]<-curr.part.mat
loglik.mat<-rbind(loglik.mat, curr.loglik.vec)
}

list(curr.part.mat, curr.weight.vec, curr.loglik.vec, NC.vec, "Resample times"=resample.vec)
#curr.part.mat=list of mutated samples associated with the final iteration
#curr.weight.vec=the associated weights
#curr.loglik.vec=the associated loglik
#NC.vec=vector of normalising constants concerned with normalising constants of weights.
# "Resample times"=resample.vec =iterations at which we resampled.
}

